/* ========================================================================= */
/* HELPERS ================================================================= */
/* ========================================================================= */

getDevice = function() {
    return (
        window.getComputedStyle(
            document.querySelector('html'), ':before'
        ).getPropertyValue('content')
        .replace(/["']/g, '') // Fix for: "'mobile'", ""mobile""...
    );
};

trimInput = function(value) {
	return value.replace(/^\s*|\s*$/g, '');
};

/* ========================================================================= */
/* FUNCTIONS =============================================================== */
/* ========================================================================= */

var siteHandler = {
	/**
	 * Carga un template con los datos que recibe como parámetro.
	 *
	 * @param  {String}   name    	- Nombre del fichero .hbs donde se encuentra el template
	 * @param  {String}   where   	- Selector CSS donde vamos a insertar el template
	 * @param  {Object}	  context	- Objeto con los datos que va a utilizar el template
	 * @param  {Boolean}  append  	- Si es true inserta el template al final de un contenedor para no borrar su contenido
	 * @param  {Function} callback  - Función de callback
	 */
	loadTemplate: function(name, where, context, append, func) {
		var location = where || '#' + name,
			data = context,
			template = JST['app/templates/' + name + '.hbs'],
			html = template(data),
			callback = func;

		if (append)
			$(location).append(html);
		else
			document.querySelector(location).innerHTML = html;
		if (func)
			callback();
	},
	/**
	 * Show/hide animals
	 *
	 * @param  {node} element - The element that fires the event
	 */
	handleClick: function(element) {
		var $this    = $(element),
			$animals = $this.find('.animals'),
			name	 = $this.data('name');

		// Warning: transitionend fires for each property transitioned!
		$animals.one("webkitTransitionEnd otransitionend oTransitionEnd msTransitionEnd transitionend", function(event) {
			// Do something when the transition ends
			if ($animals.hasClass('closed')) {
				$animals.hide();

				// Enable click for current element again (placed here because of 'else' setTimeout)
				$this.one('click', function(e) {
					siteHandler.handleClick(e.currentTarget);
				});
			}
			else {
				var list  = $animals.find('img'),
					title = $animals.find('.recipe-title'),
					catTitle = $animals.find('.category-title');
				_.each(list, function(value, index, list){
					// $animals.find('img, span').delay(500).animate({ opacity: 1 }, function(){});
					var $img  = $(value),
						$span = $(value).next();

					$img.delay(500+(100*index)).animate({ opacity: 1 }, function(){});
					$span.delay(500+(100*index)).animate({ opacity: 1 }, function(){});
					title.delay(500+(100*index)).animate({ opacity: 1 }, function(){});
					catTitle.delay(500+(100*index)).animate({ opacity: 1 }, function(){});

					if (index == list.length -1) {
						setTimeout(function() {
							$this.addClass('done');

							// Enable click for current element again (placed here because of setTimeout)
							$this.one('click', function(e) {
								siteHandler.handleClick(e.currentTarget);
							});
						}, 600+(100*index));
					}
				});
			}
		});

		if ($animals.hasClass('closed')) {
			$animals.show(400, function() {
				// Throws transition end event
				$animals.removeClass('closed');
			});
		}
		else if ($this.hasClass('done')) {
			$animals.find('img, span').animate({ opacity: 0 }, function(){
				// Throws transition end event
				$animals.addClass('closed');
				$this.removeClass('done');
			});
		}
	},
	/**
	 * Establece los eventos de la app
	 */
	setEvents: {
		activeCategory: function() {
			$('.category').on('click', function(e) {
				// Disable click for current element (will be enabled when transition ends)
				$(e.currentTarget).off('click');
				// Act on the event
				siteHandler.handleClick(e.currentTarget);
			});
			$('.animal-card').on('click', function(event) {
				event.preventDefault();
				event.stopPropagation();
				/* Act on the event */
			});
			$('.dish-card').on('click', function(event) {
				event.preventDefault();
				event.stopPropagation();
				/* Act on the event */
				$('.modal-body').addClass('padding30');
				siteHandler.loadTemplate($(this).data('modal'),'#nucleusModal .modal-body');
			    $('#nucleusModal').modal();
			});
			$('.recipe-title').on('click', function(event) {
				event.stopPropagation();
			});
		},
		activeLegend: function() {
			$('#curtainIcon').on('click', function(event) {
				event.preventDefault();
				$(this).find('i').toggleClass('fa-search-plus fa-search-minus');
				/* Act on the event */
				$('.legend-wrapper').toggleClass('closed');
			});
			$('#infoIcon').on('click', function(event) {
		        event.preventDefault();
		        event.stopPropagation();
		        /* Act on the event */
		        siteHandler.loadTemplate($(this).data('modal'),'#nucleusModal .modal-body');
	            $('#nucleusModal').modal();
	        });
		},
		centerModals: function(){
			function modalCenter(){
				$('.modal').each(function(i){
					$clone = $(this).clone().css('display', 'block').appendTo('body');
					var top = Math.round(($clone.height() - $clone.find('.modal-content').height()) / 2);
					top = top > 0 ? top : 0;
					$clone.remove();
					$(this).find('.modal-content').css("margin-top", top);
				});
			}
			$('.modal').on('show.bs.modal', modalCenter);
			$(window).on('resize', modalCenter);
		},
		/**
		 * Inserta tantos div.animal-card como sean necesarios para que los ingredientes
		 * sueltos queden alineados a la izquierda, es decir, incrementa el número de
		 * ingredientes para que sean multiplo del número que recibe como parámetro.
		 *
		 * @param  {int} num  - Número de ingredientes por fila
		 */
		alignIngredientsToLeft: function(num) {
			var $ingredients = $('.ingredients');

			_.each($ingredients, function (currentIngredients) {
				var total = $(currentIngredients).find('.animal-card').length,
					increment = num - (total % num);

				for (var i=0; i < increment; i++) {
					$(currentIngredients).append('<div class="animal-card fix-align-left"></div>');
				}
			});
		}
	},
	init: function() {
		//load template layout (header, grid, and site-canvas)
		this.loadTemplate('layout', '#mainWrapper', null, false, function(){
			siteHandler.loadTemplate('main');
			siteHandler.setEvents.activeCategory();
			siteHandler.setEvents.activeLegend();
			siteHandler.setEvents.alignIngredientsToLeft(3);
			siteHandler.setEvents.centerModals();
		});
	}
};

siteHandler.init();
